"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/strategy/Thread.ts
var import_ws2 = require("@discordjs/ws");
var import_worker_threads2 = require("worker_threads");

// src/strategy/IndomitableFetchingStrategy.ts
var import_ws = require("@discordjs/ws");

// src/Util.ts
var import_node_https = __toESM(require("https"));
var EnvProcessData = {
  clusterId: Number(process.env.INDOMITABLE_CLUSTER || 0),
  clusterCount: Number(process.env.INDOMITABLE_CLUSTER_TOTAL || 0),
  shardIds: (process.env.INDOMITABLE_SHARDS || "").split(" ").map(Number),
  shardCount: Number(process.env.INDOMITABLE_SHARDS_TOTAL || 0)
};

// src/strategy/IndomitableFetchingStrategy.ts
var IndomitableFetchingStrategy = class {
  constructor(ipc2, options2) {
    this.ipc = ipc2;
    this.options = options2;
  }
  async retrieveSessionInfo(shardId) {
    const content = {
      op: "retrieveSession" /* RETRIEVE_SESSION */,
      event: import_ws.WebSocketShardEvents.Ready,
      data: { shardId },
      shardId,
      internal: true
    };
    return await this.ipc.send({ content, repliable: true });
  }
  async updateSessionInfo(shardId, sessionInfo) {
    const content = {
      op: "updateSession" /* UPDATE_SESSION */,
      event: import_ws.WebSocketShardEvents.Ready,
      data: { shardId, sessionInfo },
      shardId,
      internal: true
    };
    await this.ipc.send({ content });
  }
  async waitForIdentify(shardId, signal) {
    const content = {
      op: "requestIdentify" /* REQUEST_IDENTIFY */,
      event: import_ws.WebSocketShardEvents.Ready,
      data: { shardId },
      shardId,
      internal: true
    };
    const listener = () => this.abortIdentify(shardId);
    try {
      signal.addEventListener("abort", listener);
      await this.ipc.send({ content, repliable: true });
    } finally {
      signal.removeEventListener("abort", listener);
    }
  }
  abortIdentify(shardId) {
    const content = {
      op: "cancelIdentify" /* CANCEL_IDENTIFY */,
      event: import_ws.WebSocketShardEvents.Ready,
      data: { shardId },
      shardId,
      internal: true
    };
    this.ipc.send({ content, repliable: false }).catch(() => null);
  }
};

// src/ipc/ThreadStrategyWorker.ts
var import_node_events = __toESM(require("events"));
var import_worker_threads = require("worker_threads");

// src/ipc/BaseIpc.ts
var import_crypto = require("crypto");
var BaseIpc = class {
  constructor(manager) {
    this.manager = manager;
    this.promises = /* @__PURE__ */ new Map();
  }
  /**
   * Number of promises pending to be resolved
   */
  get pendingPromises() {
    return this.promises.size;
  }
  /**
   * Rejects all the pending promises
   */
  flushPromises(reason) {
    const error = new Error(reason);
    for (const promise of this.promises.values()) {
      if (promise.controller) {
        promise.controller.signal.removeEventListener("abort", promise.controller.listener);
      }
      promise.reject(error);
    }
    this.promises.clear();
  }
  /**
   * Raw send method without abort controller handling
   * @param transportable Data to send
   */
  send(transportable) {
    return new Promise((resolve, reject) => {
      if (!this.available()) {
        this.manager.emit("debug" /* DEBUG */, "IPC tried to send a message, but the ipc communication is not yet ready");
        return resolve(void 0);
      }
      const repliable = transportable.repliable || false;
      const id = repliable ? (0, import_crypto.randomUUID)() : null;
      const data = {
        id,
        content: transportable.content,
        internal: true,
        type: "message" /* MESSAGE */
      };
      this.sendData(data);
      if (!id)
        return resolve(void 0);
      this.waitForPromise({ id, resolve, reject, signal: transportable.signal });
    });
  }
  /**
   * Taps into message event of worker or primary process to handle ipc communication
   * @internal
   */
  async handleRawResponse(data, errorCallback) {
    try {
      this.manager.emit("raw" /* RAW */, data);
      if (!data.internal)
        return;
      switch (data.type) {
        case "message" /* MESSAGE */:
          return await this.handleUnparsedMessage(data);
        case "response" /* RESPONSE */:
        case "error" /* ERROR */:
          return this.handlePromise(data);
      }
    } catch (error) {
      errorCallback(error);
    }
  }
  waitForPromise(options2) {
    let controller;
    if (options2.signal) {
      const listener = () => {
        this.promises.delete(options2.id);
        options2.reject(new Error("This operation is aborted"));
      };
      controller = {
        listener,
        signal: options2.signal
      };
      controller.signal.addEventListener("abort", listener);
    }
    this.promises.set(options2.id, { resolve: options2.resolve, reject: options2.reject, controller });
  }
  handlePromise(data) {
    const id = data.id;
    const promise = this.promises.get(id);
    if (!promise)
      return;
    this.promises.delete(id);
    if (promise.controller) {
      promise.controller.signal.removeEventListener("abort", promise.controller.listener);
    }
    if (data.type === "error" /* ERROR */) {
      const content = data.content;
      const error = new Error(content.reason);
      error.stack = content.stack;
      error.name = content.name;
      promise.reject(error);
      return;
    }
    promise.resolve(data.content);
  }
  async handleUnparsedMessage(data) {
    const reply = (content) => {
      if (!data.id)
        return;
      const response = {
        id: data.id,
        content,
        internal: true,
        type: "response" /* RESPONSE */
      };
      this.sendData(response);
    };
    const message = {
      repliable: !!data.id,
      content: data.content,
      reply
    };
    if (!data.content.internal)
      return this.emitMessage(message);
    try {
      await this.handleMessage(message);
    } catch (error) {
      if (!message.repliable)
        return;
      const response = {
        id: data.id,
        content: {
          name: error.name,
          reason: error.reason,
          stack: error.stack
        },
        internal: true,
        type: "error" /* ERROR */
      };
      this.sendData(response);
    }
  }
  emitMessage(message) {
    this.manager.emit("message" /* MESSAGE */, message);
  }
};

// src/ipc/ThreadStrategyWorker.ts
var ThreadStrategyWorker = class extends BaseIpc {
  constructor() {
    super(new import_node_events.default());
    import_worker_threads.parentPort.on("message", (message) => this.handleRawResponse(message, () => null));
  }
  build(shard2) {
    if (!this.shard)
      this.shard = shard2;
  }
  available() {
    return !!import_worker_threads.parentPort;
  }
  sendData(data) {
    return import_worker_threads.parentPort.postMessage(data);
  }
  async handleMessage(message) {
    const content = message.content;
    if (!this.shard)
      throw new Error("Shard isn't initialized yet");
    switch (content.op) {
      case "connect" /* CONNECT */:
        await this.shard.connect();
        message.reply(null);
        break;
      case "destroy" /* DESTROY */:
        await this.shard.destroy(content.data || {});
        message.reply(null);
        break;
      case "send" /* SEND */:
        await this.shard.send(content.data || {});
        message.reply(null);
        break;
      case "reconnect" /* RECONNECT */:
        await this.shard.destroy(content.data);
        message.reply(null);
        break;
      case "status" /* STATUS */:
        message.reply(this.shard.status);
        break;
    }
  }
};

// src/strategy/Thread.ts
var options = import_worker_threads2.workerData;
var ipc = new ThreadStrategyWorker();
var strategy = new IndomitableFetchingStrategy(ipc, options);
var shard = new import_ws2.WebSocketShard(strategy, options.shardId);
ipc.build(shard);
for (const event of Object.values(import_ws2.WebSocketShardEvents)) {
  shard.on(event, (data) => {
    const content = {
      op: "shardEvent" /* SHARD_EVENT */,
      event,
      data,
      shardId: shard.id,
      internal: true
    };
    ipc.send({ content }).catch(() => null);
  });
}
//# sourceMappingURL=Thread.js.map